#<profile4xi profile = "03" />
#<program4xi program = "_connectionbase4xi.py">
#<abstract4xi abstract = "providing class _ConnectionBase4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ '_ConnectionBase4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:  
    import errno     # system ones
    import io
    import itertools
    import os
    import pickle
    import select
    import signal
    import socket
    import struct
    import sys
    import tempfile
    import time
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
    
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None

                
try:                 # rare ones
    pass    
except:
    pass
import pickle     
try:                 # imported-as ones 
    from pickle import _Pickler as Pickler         
except:
    pass 
               
   
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
try:                             # Python ones
    from multiprocessing.connection import _ConnectionBase
except:
    print("_connectionbase4xi.py *** failed in from multiprocessing.connection import _ConnectionBase ***") 
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from current_process4xi import get_authkey_current_process4xi
from forkingpickler4xi import ForkingPickler4Xi

from register_after_fork4xi import register_after_fork4xi
from sub_warning4xi import sub_warning4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
#"
#/>

# =================================================================================================
#<class4xi class = "_ConnectionBase4Xi" base = "object" child = "None" synopsis = "_ConnectionBase4Xi class" usage = "
#   
#>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class _ConnectionBase4Xi(object): 
    '''
    _ConnectionBase4Xi class
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('fileno', 'close', 'send_bytes', 'send', 'sense', 'recv_bytes', 'recv_bytes_into', 'recv', 'poll', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _handle = None
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ConnectionBase4Xi" usage = "initializes _ConnectionBase4Xi with (self, handle, readable=True, writable=True)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, handle, readable=True, writable=True):
        '''
        initializes _ConnectionBase4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "_ConnectionBase4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/4a) started" + "; init-of-ConnectionBase4Xi-started: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        
        message2print = "{(Step 1/4b) borning" + "; connectionbase2xi-of-ConnectionBase4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        
        message2print = "{(Step 1/4c) inheriting" + "; object-inheriting-by-connectionbase2xi-of-ConnectionBase4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HP: <class 'object'>
        
        message2print = "{(Step 1/4d) passed" + "; handle-passed-to-init-of-ConnectionBase4Xi: " + str(handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: 256
        
        message2print = "{(Step 1/4e) passed" + "; readable-passed-to-init-of-ConnectionBase4Xi: " + str(readable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: True
         
        message2print = "{(Step 1/4f) passed" + "; writable-passed-to-init-of-ConnectionBase4Xi: " + str(writable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: True
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------    
        handle2xi = None
        try:
            handle2xi = handle.__index__()
            message2print = "{(Step 2/4) set" + "; handle2xi-set-by-init-of-ConnectionBase4Xi to: " + str(handle2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HP: 256
            
            if handle2xi < 0:
                raise ValueError("init-of-ConnectionBase4Xi - invalid handle")
            if not readable and not writable:
                raise ValueError(
                    "init-of-ConnectionBase4Xi - at least one of `readable` and `writable` must be True")
        except:
            message2print = "{(Step 2/4z) yielded" + "; handle2xi-yielded-by-init-of-ConnectionBase4Xi: " + str(handle2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------    
        self._handle = handle2xi
        message2print = "{(Step 3/4a) set" + "; handle-of-connectionbase2xi-set-by-init-of-ConnectionBase4Xi to: " + str(handle2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HP: 256
        
        self._readable = readable
        message2print = "{(Step 3/4b) set" + "; readable-of-connectionbase2xi-set-by-init-of-ConnectionBase4Xi to: " + str(self._readable)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HP: True
        
        
        self._writable = writable
        message2print = "{(Step 3/4c) set" + "; writable-of-connectionbase2xi-set-by-init-of-ConnectionBase4Xi to: " + str(self._writable)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HP: True
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4) 
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/4a) activated" + "; connection2xi-of-ConnectionBase4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: 03:20:16 02/18/13 Pacific Standard Time        
        
        message2print = "{(Step 4/4b) inherited" + "; object-inherited-by-connection2xi-of-ConnectionBase4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #HP: <class 'object'>   
                 
        message2print = "{(Step 4/4c) returning" + "; init-of-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP: 03:20:16 02/18/13 Pacific Standard Time                
                
    #</methodize4xi>
 
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "del-of-ConnectionBase4Xi" usage = "deletes _ConnectionBase4Xi class objects">
    #<note4xi note = "Warning Due to the precarious circumstances under which __del__() methods are invoked, 
    #                 exceptions that occur during their execution are ignored, and a warning is printed to sys.stderr instead
    #                 [Rel 7.21]
    #"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __del__(self):
        '''
        method __del__: deletes _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = '__del__'
        
           
        message2print = "{(Step 1/5) started" + "; del-of-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 05:22:44 01/08/13 Pacific Standard Time   
        #
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        try:
            if self._handle is not None:
                message2print = "{(Step 2/3a) performing" + "; close-performing-by-del-of-ConnectionBase4Xi: " + str(self._close)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #HP: <bound method PipeConnection4Xi._close of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000002BBC240>> 
                #HP: <bound method PipeConnection4Xi._close of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
                try:    
                    self._close()
                    message2print = "{(Step 2/3b) performed" + "; close-performed-by-del-of-ConnectionBase4Xi: " + str(self._close)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 2/3z1) failed" + "; close-failed-by-del-of-ConnectionBase4Xi: " + str(None)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #     
            else:
                message2print = "{(Step 2/3b) warned" + "; handle-of-connectionbase2xi-warned-by-init-of-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
        except:
            message2print = "{(Step 2/3z1) failed" + "; handle-of-connectionbase2xi-failed-by-del-of-ConnectionBase4Xi: " + str(None)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                    
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; del-of-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP: 05:22:44 01/08/13 Pacific Standard Time 
        #HP: 05:22:44 01/08/13 Pacific Standard Time
          
        return
        
    #</methodize4xi>    
               
               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "check_closed-ConnectionBase4Xi" usage = "checks closing for _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _check_closed(self):
        '''
        method _check_closed: checks closing for _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = '_check_closed'
        
        message2print = "{(Step 1/3) started" + "; check_closed-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        #TB: 08:29:13 02/18/13 Pacific Standard Time
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        if self._handle is None:
            message2print = "{(Step 2/3a) yielded" + "; handle-yielded-by-check_closed-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            raise IOError("check_closed-of-ConnectionBase4Xi: handle is None i.e. already closed")
        else:
            message2print = "{(Step 2/3b) using" + "; handle-using-by-check_closed-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HP: 256  
            #TB: 412      
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; check_closed-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        #TB: 08:29:13 02/18/13 Pacific Standard Time
          
        return
        
    #</methodize4xi>
    
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "check_readable-ConnectionBase4Xi" usage = "checks whether object is readable for _ConnectionBase4Xi">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _check_readable(self):
        '''
        method _check_readable: checks whether object is readable for _ConnectionBase4Xi
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = '_check_readable'
        
           
        message2print = "{(Step 1/3) started" + "; check_readable-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        #TB: 08:29:13 02/18/13 Pacific Standard Time
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        if not self._readable:
            message2print = "{(Step 1/3z) yielded" + "; connection-yielded-by-check_readable-ConnectionBase4Xi: " + " connection is write-only ; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            raise IOError("check_readable-ConnectionBase4Xi: connection is write-only")
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 3/3) returning" + "; check_readable-ConnectionBase4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #HP: 03:20:16 02/18/13 Pacific Standard Time
            #TB: 08:29:13 02/18/13 Pacific Standard Time
              
            return
        
    #</methodize4xi>    
                                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "check_writable-ConnectionBase4Xi" usage = "checks whether writable for _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _check_writable(self):
        '''
        method _check_writable: checks whether writable for _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = '_check_writable'
        
           
        message2print = "{(Step 1/3) started" + "; check_writable-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 03:20:16 02/18/13 Pacific Standard Time
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        if not self._writable:
            message2print = "{(Step 2/3z) yielded" + "; connection-yielded-by-check_writable-ConnectionBase4Xi: " + " connection is read-only ; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #      
            
            raise IOError("check_writable-ConnectionBase4Xi: connection is read-only")
              
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 3/3) returning" + "; check_writable-ConnectionBase4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #HP: 03:20:16 02/18/13 Pacific Standard Time 
              
            return
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "bad_message_length-ConnectionBase4Xi" usage = "checks message_length for _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _bad_message_length(self):
        '''
        method _bad_message_length: checks message_length for _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = '_bad_message_length'
        
           
        message2print = "{(Step 1/3) started" + "; bad_message_length-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        if self._writable:
            self._readable = False
            message2print = "{(Step 2/3a) set" + "; readable-of-connectionbase2xi-set-by-bad_message_length-ConnectionBase4Xi to: " + str(self._readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        else:
            self.close()
            message2print = "{(Step 2/3b) performed" + "; close-of-connectionbase2xi-performed-by-bad_message_length-ConnectionBase4Xi: " + str(self.close)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        
        message2print = "{(Step 2/3z) yielded" + "; connection-yielded-by-bad_message_length-ConnectionBase4Xi: " + " bad message length ; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #      
                    
        raise IOError("bad_message_length-ConnectionBase4Xi: bad message length")

        
    #</methodize4xi>
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "closed-property-of-ConnectionBase4Xi" usage = "depicts whether connection is closed">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    @property
    def closed(self):
        '''
        method closed: depicts whether connection is closed
        '''
        return self._handle is None
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "readable-property-of-ConnectionBase4Xi" usage = "depicts whether connection is readable">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    @property
    def readable(self):
        '''
        method readable: depicts whether connection is readable
        '''
        return self._readable
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "writable-property-of-ConnectionBase4Xi" usage = "depicts whether connection is writable">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    @property
    def writable(self):
        '''
        method writable: depicts whether connection is writable
        '''
        return self._writable
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "fileno-ConnectionBase4Xi" usage = "returns file descriptor or handle of the connection">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def fileno(self):
        '''
        method fileno: returns file descriptor or handle of the connection
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = 'fileno'
        
           
        message2print = "{(Step 1/3) started" + "; fileno-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------  
        try:
            self._check_closed()
            
            message2print = "{(Step 2/3) performed" + "; close-of-connectionbase2xi-performed-by-fileno-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
        except:
            message2print = "{(Step 2/3z) failed" + "; closed-failed-by-check_writable-ConnectionBase4Xi: " + str(None)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                    
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; handle-of-connectionbase2xi-returning-by-fileno-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
          
        return self._handle
        
    #</methodize4xi> 
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "close-ConnectionBase4Xi" usage = "closes the connection">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def close(self):
        '''
        method close: closes the connection
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = 'close'
        
           
        message2print = "{(Step 1/3) started" + "; close-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        if self._handle is not None:
            try:   
                # -----------------------------------------------------------------------------------------
                # (Step 2/3)
                # -----------------------------------------------------------------------------------------  
                self._close()
                message2print = "{(Step 2/3a) performed" + "; close-of-connectionbase2xi-performed-by-close-ConnectionBase4Xi: " + str(self._close)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            finally:
                self._handle = None
                message2print = "{(Step 2/3b) set" + "; handle-of-connectionbase2xi-set-by-close-ConnectionBase4Xi to: " + str(self._handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
        else:
            message2print = "{(Step 2/3c) got" + "; handle-of-connectionbase2xi-got-by-close-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                 
                
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; close-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
          
        return
        
    #</methodize4xi> 


                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "send_bytes-ConnectionBase4Xi" usage = "send bytes via connection">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def send_bytes(self, buf, offset=0, size=None):
        '''
        method send_bytes: send bytes via connection
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/9)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'send_bytes'
        
        message2print = "{(Step 1/9a) started" + "; send_bytes-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 05:22:44 01/08/13 Pacific Standard Time
        
        message2print = "{(Step 1/9b) passed" + "; buf-passed-to-send_bytes-ConnectionBase4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: b'(STEP 4/33) OF HELLO DUPLEX PIPE4XI: ROGER A4XI, B4XI HERE !' 
        #HP: array('i', [0, 1, 2, 3, 4] 
        #HP: b'(STEP 8A/33) OF HELLO DUPLEX PIPE4XI: ARRAY RECEIVED OK, ROGER FROM B4XI TO A4XI!'
        #HP: array('i', [0, 1, 2, 3, 4])
        
        message2print = "{(Step 1/9c) passed" + "; offset-passed-to-send_bytes-ConnectionBase4Xi: " + str(offset) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: 0
        
        message2print = "{(Step 1/9d) passed" + "; size-passed-to-send_bytes-ConnectionBase4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: None
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/9)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_closed()
            message2print = "{(Step 2/9a) performed" + "; check_closed-performed-by-send_bytes-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
        except:
            message2print = "{(Step 2/9z) yielded" + "; handle-yielded-by-send_bytes-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
           
        # -----------------------------------------------------------------------------------------
        # (Step 3/9)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_writable()
            message2print = "{(Step 3/9a) performed" + "; check_writable-performed-by-send_bytes-ConnectionBase4Xi: " + str(self._check_writable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_writable of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
        except:
            message2print = "{(Step 3/9z) yielded" + "; handle-yielded-by-send_bytes-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                    
        # -----------------------------------------------------------------------------------------
        # (Step 4/9)      
        # -----------------------------------------------------------------------------------------
        m2xi = None
        ack2memory = False
        try:
            m2xi = memoryview(buf)
            message2print = "{(Step 4/9a) set" + "; m2xi-set-by-send_bytes-ConnectionBase4Xi to: " + str(m2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <memory at 0x00000000036872A0>
            
            ack2memory = True
        except:
            message2print = "{(Step 4/9z) yielded" + "; m2xi-yielded-by-send_bytes-ConnectionBase4Xi: " + str(m2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/9) - HACK for byte-indexing of non-bytewise buffers (e.g. array.array)      
        # -----------------------------------------------------------------------------------------
        if ack2memory is True:
            if m2xi.itemsize > 1:
                try:
                    m2xi = memoryview(bytes(m2xi))
                    message2print = "{(Step 5/9a) set" + "; m2xi-set-by-send_bytes-ConnectionBase4Xi to: " + str(m2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    #
                except:
                    message2print = "{(Step 5/9z1) yielded" + "; m2xi-yielded-by-send_bytes-ConnectionBase4Xi: " + str(m2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    #
            else:
                message2print = "{(Step 5/9b) using" + "; itemsize-using-by-send_bytes-ConnectionBase4Xi: " + str(m2xi.itemsize)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 1
                             
        else:
            message2print = "{(Step 5/9z2) yielded" + "; m2xi-yielded-by-send_bytes-ConnectionBase4Xi: " + str(m2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
        
        # -----------------------------------------------------------------------------------------
        # (Step 6/9)      
        # -----------------------------------------------------------------------------------------
        if ack2memory is True:
            n2xi = len(m2xi)
            message2print = "{(Step 6/9a) set" + "; n2xi-set-by-send_bytes-ConnectionBase4Xi to: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 60; 20 for array; 81 for next
        else:
            n2xi = 0
            message2print = "{(Step 6/9z1) yielded" + "; n2xi-yielded-by-send_bytes-ConnectionBase4Xi: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
            
                
        if offset < 0:
            message2print = "{(Step 6/9z2) warned" + "; offset-warned-by-send_bytes-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            raise ValueError("send_bytes-ConnectionBase4Xi: offset is negative")
            
        if n2xi < offset:
            message2print = "{(Step 6/9z3) warned" + "; n2xi-warned-by-send_bytes-ConnectionBase4Xi: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
            
            raise ValueError("send_bytes-ConnectionBase4Xi: buffer length < offset")
            
            
        # -----------------------------------------------------------------------------------------
        # (Step 7/9)      
        # -----------------------------------------------------------------------------------------
        if size is None:
            message2print = "{(Step 7/9a) using" + "; n2xi-using-by-send_bytes-ConnectionBase4Xi: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 60; 81
            
            message2print = "{(Step 7/9b) using" + "; offset-using-by-send_bytes-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 0; 0 
            size = n2xi - offset
            
            message2print = "{(Step 7/9c) set" + "; size-set-by-send_bytes-ConnectionBase4Xi to: " + str(size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 60; 81
             
        elif size < 0:
            message2print = "{(Step 7/9z1) warned" + "; size-warned-by-send_bytes-ConnectionBase4Xi: " + str(size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
        
            raise ValueError("send_bytes-ConnectionBase4Xi: size is negative")
        elif offset + size > n2xi:
            message2print = "{(Step 7/9z2) warned" + "; offset+size-warned-by-send_bytes-ConnectionBase4Xi: " + str(offset + size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
            message2print = "{(Step 7/9z2) warned" + "; n2xi-warned-by-send_bytes-ConnectionBase4Xi: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
                    
            raise ValueError("send_bytes-ConnectionBase4Xi: buffer length < offset + size")
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 8/9)      
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 8/9a) using" + "; m2xi-using-by-send_bytes-ConnectionBase4Xi: " + str(m2xi[offset:offset + size])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: <memory at 0x0000000003687430>
  
        try:
            self._send_bytes(m2xi[offset:offset + size])
    
            message2print = "{(Step 8/9b) performed" + "; send-performed-by-send_bytes-ConnectionBase4Xi: " + str(self._send_bytes)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._send_bytes of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
            # 
        except:
            message2print = "{(Step 8/9z1) failed" + "; send_bytes-failed-by-send_bytes-ConnectionBase4Xi: " + str(self._send_bytes)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            message2print = "{(Step 8/9z2) failed" + "; m2xi-failed-by-send_bytes-ConnectionBase4Xi: " + str(m2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            message2print = "{(Step 8/9z3) failed" + "; n2xi-failed-by-send_bytes-ConnectionBase4Xi: " + str(n2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            message2print = "{(Step 8/9z4) failed" + "; offset-failed-by-send_bytes-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            message2print = "{(Step 8/9z5) failed" + "; size-failed-by-send_bytes-ConnectionBase4Xi: " + str(size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            

                    
        # -----------------------------------------------------------------------------------------
        # (Step 9/9)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 9/9) returning" + "; send_bytes-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP: case: (STEP 6A/33) OF HELLO DUPLEX PIPE4XI - SENT ARR1:  array('i', [0, 1, 2, 3, 4])
          
        return
    #</methodize4xi>        
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "send-ConnectionBase4Xi" usage = "sends a pickable object via connection">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def send(self, obj):
        '''
        method send: sends a pickable object via connection
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/7)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'send'
        
        message2print = "{(Step 1/7a) started" + "; send-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        
        message2print = "{(Step 1/7b) passed" + "; obj-passed-to-send-ConnectionBase4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: [1, '(STEP 2A/33) OF HELLO DUPLEX PIPE4XI: HELLO B4XI, A4XI HERE !', None] 
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/7)      
        # -----------------------------------------------------------------------------------------        
        try:
            self._check_closed()
            message2print = "{(Step 2/7a) performed" + "; check_closed-performed-by-send-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035C10F0>>
        except:
            message2print = "{(Step 2/7z) yielded" + "; handle-yielded-by-send-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
           
        # -----------------------------------------------------------------------------------------
        # (Step 3/7)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_writable()
            message2print = "{(Step 3/7a) performed" + "; check_writable-performed-by-send-ConnectionBase4Xi: " + str(self._check_writable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_writable of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035C10F0>>
        except:
            message2print = "{(Step 3/7z) yielded" + "; handle-yielded-by-send-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                    
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/7)      
        # -----------------------------------------------------------------------------------------
        buf2xi = None
        try:
            buf2xi = io.BytesIO()
            message2print = "{(Step 4/9a) set" + "; buf2xi-set-by-send-ConnectionBase4Xi to: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <_io.BytesIO object at 0x00000000037E4648>
        except:
            message2print = "{(Step 4/7z) yielded" + "; buf2xi-yielded-by-send-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
            
        # -----------------------------------------------------------------------------------------
        # (Step 5/7) - HACK for byte-indexing of non-bytewise buffers (e.g. array.array)      
        # -----------------------------------------------------------------------------------------
        try:
            ForkingPickler4Xi(buf2xi, pickle.HIGHEST_PROTOCOL).dump(obj)
            message2print = "{(Step 5/7a) performed" + "; ForkingPickler4Xi-performed-by-send-ConnectionBase4Xi: " + str(ForkingPickler4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <class 'forkingpickler4xi.ForkingPickler4Xi'>         
        except:
            message2print = "{(Step 5/7z) yielded" + "; ForkingPickler4Xi-yielded-by-send-ConnectionBase4Xi: " + str(ForkingPickler4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #                     
            
        # -----------------------------------------------------------------------------------------
        # (Step 6/7)      
        # -----------------------------------------------------------------------------------------
        buffer2xi = None
        try:
            buffer2xi = buf2xi.getbuffer()
            message2print = "{(Step 6/7a) got" + "; buffer2xi-got-by-send-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <memory at 0x00000000038BAE58>        
            
        except:
            message2print = "{(Step 6/7z1) yielded" + "; buffer2xi-yielded-by-send-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #               
        try:    
            self._send_bytes(buffer2xi)
    
            message2print = "{(Step 6/7b) performed" + "; send-performed-by-send-ConnectionBase4Xi: " + str(self._send_bytes)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._send_bytes of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035C10F0>>
        except:
            message2print = "{(Step 6/7z2) failed" + "; send-failed-by-send-ConnectionBase4Xi: " + str(self._send_bytes)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                       
            message2print = "{(Step 6/7z3) failed" + "; buffer2xi-failed-by-send-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #              
        
        # -----------------------------------------------------------------------------------------
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 7/7) returning" + "; send-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HP: 03:20:16 02/18/13 Pacific Standard Time
        return
        
    #</methodize4xi>    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "recv_bytes-ConnectionBase4Xi" usage = "receives bytes data as a bytes object">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def recv_bytes(self, maxlength=None):
        '''
        method recv_bytes: receives bytes data as a bytes object
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/7)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'recv_bytes'
        
        message2print = "{(Step 1/7a) started" + "; recv_bytes-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 05:46:47 01/22/13 Pacific Standard Time
        
        message2print = "{(Step 1/7b) passed" + "; maxlength-passed-to-recv_bytes-ConnectionBase4Xi: " + str(maxlength) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: None
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/7)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_closed()
            message2print = "{(Step 2/7a) performed" + "; check_closed-performed-by-recv_bytes-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003525F28>>
        except:
            message2print = "{(Step 2/7z) yielded" + "; handle-yielded-by-recv_bytes-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
           
        # -----------------------------------------------------------------------------------------
        # (Step 3/7)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_readable()
            message2print = "{(Step 3/7a) performed" + "; check_readable-performed-by-recv_bytes-ConnectionBase4Xi: " + str(self._check_readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_readable of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003525F28>> 
        except:
            message2print = "{(Step 3/7z) yielded" + "; handle-yielded-by-recv_bytes-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                    
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/7)      
        # -----------------------------------------------------------------------------------------
        if maxlength is not None and maxlength < 0:
            message2print = "{(Step 4/7z1) yielded" + "; maxlength-yielded-by-recv_bytes-ConnectionBase4Xi: " + str(maxlength)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            maxlength = abs(maxlength)
            
            message2print = "{(Step 4/7a) set" + "; maxlength-set-by-recv_bytes-ConnectionBase4Xi to: " + str(maxlength)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 

            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------
        buf2xi = None
        try:
            buf2xi = self._recv_bytes(maxlength)
            message2print = "{(Step 5/7a) got" + "; buf2xi-got-by-recv_bytes-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <_io.BytesIO object at 0x000000000378F348>
        except: 
            message2print = "{(Step 5/7z) yielded" + "; buf2xi-yielded-by-recv_bytes-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #              
        
        # -----------------------------------------------------------------------------------------
        # (Step 6/7)      
        # -----------------------------------------------------------------------------------------
        if buf2xi is None:
            self._bad_message_length()

            message2print = "{(Step 6/7a) performed" + "; bad_message_length-performed-by-recv_bytes-ConnectionBase4Xi: " + str(self._bad_message_length)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        else:
            message2print = "{(Step 6/7b) using" + "; buf2xi-using-by-recv_bytes-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <_io.BytesIO object at 0x000000000378F348>             
        
        # -----------------------------------------------------------------------------------------
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------        
        value2xi = None
        try:
            value2xi = buf2xi.getvalue()
            
            message2print = "{(Step 7/7a) got" + "; value2xi-got-by-recv_bytes-ConnectionBase4Xi: " + str(value2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: b'(STEP 4/33) OF HELLO DUPLEX PIPE4XI: ROGER A4XI, B4XI HERE !'
            
        except:
            message2print = "{(Step 7/7b) yielded" + "; value2xi-yielded-by-recv_bytes-ConnectionBase4Xi: " + str(value2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
        
        message2print = "{(Step 7/7c) returning" + "; value2xi-returning-by-recv_bytes-ConnectionBase4Xi: " + str(value2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: b'(STEP 4/33) OF HELLO DUPLEX PIPE4XI: ROGER A4XI, B4XI HERE !'
                  
        return value2xi
    #</methodize4xi>    

                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "recv_bytes_into-ConnectionBase4Xi" usage = "receive bytes data into a writeable buffer-like object">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def recv_bytes_into(self, buf, offset=0):
        '''
        method recv_bytes_into: receives bytes data into a writeable buffer-like object and returns the number of bytes read
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/9)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'recv_bytes_into'
        
        message2print = "{(Step 1/9a) started" + "; recv_bytes_into-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/9b) passed" + "; buf-passed-to-recv_bytes_into-ConnectionBase4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: array('i', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
        
        message2print = "{(Step 1/9c) passed" + "; offset-passed-to-recv_bytes_into-ConnectionBase4Xi: " + str(offset) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: 0

        # -----------------------------------------------------------------------------------------
        # (Step 2/9)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_closed()
            message2print = "{(Step 2/9a) performed" + "; check_closed-performed-by-recv_bytes_into-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
        except:
            message2print = "{(Step 2/9z) yielded" + "; handle-yielded-by-recv_bytes_into-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                   
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/9)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_readable()
            message2print = "{(Step 2/9a) performed" + "; check_readable-performed-by-recv_bytes_into-ConnectionBase4Xi: " + str(self._check_readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_readable of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000035F4400>>
        except:
            message2print = "{(Step 2/9z) yielded" + "; handle-yielded-by-recv_bytes_into-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                               
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/9)      
        # -----------------------------------------------------------------------------------------
        with memoryview(buf) as m2xi:
            message2print = "{(Step 4/9a) using" + "; m2xi-using-by-recv_bytes_into-ConnectionBase4Xi: " + str(m2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <memory at 0x00000000036879A8> 

            
            # -----------------------------------------------------------------------------------------
            # Get bytesize of arbitrary buffer
            # -----------------------------------------------------------------------------------------
            itemsize2xi = m2xi.itemsize
            message2print = "{(Step 4/9b) got" + "; itemsize2xi-got-by-recv_bytes_into-ConnectionBase4Xi: " + str(itemsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 4
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            bytesize2xi = itemsize2xi * len(m2xi)
            message2print = "{(Step 4/9c) set" + "; bytesize2xi-set-by-recv_bytes_into-ConnectionBase4Xi to: " + str(bytesize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 40 
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            if offset < 0:
                message2print = "{(Step 4/9z1) warned" + "; offset-warned-by-recv_bytes_into-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                raise ValueError("recv_bytes_into-ConnectionBase4Xi: negative offset")
                
            elif offset > bytesize2xi:
                message2print = "{(Step 4/9z2) warned" + "; offset-warned-by-recv_bytes_into-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                
                raise ValueError("recv_bytes_into-ConnectionBase4Xi. offset too large")
             
            # -----------------------------------------------------------------------------------------
            # (Step 5/9)   
            # -----------------------------------------------------------------------------------------       
            result2xi = None
            try:
                result2xi = self._recv_bytes()
                message2print = "{(Step 5/9) got" + "; result2xi-got-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
            except:
                message2print = "{(Step 5/9z2) yielded" + "; result2xi-yielded-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                
             
            # -----------------------------------------------------------------------------------------
            # (Step 6/9)   
            # -----------------------------------------------------------------------------------------                
            size2xi = 0
            if result2xi is None:
                message2print = "{(Step 6/9z1) yielded" + "; size2xi-yielded-by-recv_bytes_into-ConnectionBase4Xi: " + str(size2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
            else:                
            
                size2xi = result2xi.tell()              # size2xi = size of received data
                message2print = "{(Step 6/9a) got" + "; size2xi-got-by-recv_bytes_into-ConnectionBase4Xi: " + str(size2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                #HP: 20 
            
                if bytesize2xi < offset + size2xi:      # bytesize2xi = size of memorybuffer in bytes - therefore offset + size2xi can not exceed memorybuffer
                    message2print = "{(Step 6/9z2) warned" + "; bytesize2xi-warned-by-recv_bytes_into-ConnectionBase4Xi: " + str(bytesize2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    #
                    message2print = "{(Step 6/9z3) warned" + "; offset-warned-by-recv_bytes_into-ConnectionBase4Xi: " + str(offset)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    #
                    message2print = "{(Step 6/9z4) warned" + "; size2xi-warned-by-recv_bytes_into-ConnectionBase4Xi: " + str(size2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    # 
                    
                    raise BufferTooShort4Xi(result2xi.getvalue())                       
                    
             
            # -----------------------------------------------------------------------------------------
            # (Step 7/9) MESSAGE CAN FIT IN DESTINATION
            # -----------------------------------------------------------------------------------------                    
            try:
                result2xi.seek(0)
                message2print = "{(Step 7/9a) performed" + "; seek-performed-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi.seek)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                #HP: <built-in method seek of _io.BytesIO object at 0x0000000003580F48>
            except:
                message2print = "{(Step 7/9z1) failed" + "; result2xi-failed-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                    
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 8/9)   
            # -----------------------------------------------------------------------------------------
            try:
                result2xi.readinto(m2xi[offset // itemsize2xi :
                                  (offset + size2xi) // itemsize2xi])
                                  
                message2print = "{(Step 8/9a) performed" + "; readinto-performed-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi.readinto)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                #HP: <built-in method readinto of _io.BytesIO object at 0x0000000003580F48>                   
            except:
                message2print = "{(Step 8/9z1) failed" + "; result2xi-failed-by-recv_bytes_into-ConnectionBase4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                            
        # -----------------------------------------------------------------------------------------
        # (Step 9/9)
        # -----------------------------------------------------------------------------------------        
       
        message2print = "{(Step 9/9) returning" + "; size2xi-returning-by-recv_bytes_into-ConnectionBase4Xi: " + str(size2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: 20 for array('i', [0, 1, 2, 3, 4, 0, 0, 0, 0, 0])
          
        return size2xi
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "recv-ConnectionBase4Xi" usage = "receives a (picklable) object">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def recv(self):
        '''
        method recv: receives a (picklable) object
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'recv'
        
        message2print = "{(Step 1/5a) started" + "; recv-ConnectionBase4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #HP: 01:53:56 02/19/13 Pacific Standard Time
        #TB: 08:29:13 02/18/13 Pacific Standard Time <-- test_ball4xi.py

        # -----------------------------------------------------------------------------------------
        # (Step 2/5)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_closed()
            message2print = "{(Step 2/5a) performed" + "; check_closed-performed-by-recv-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000037E6390>>
            #TB: <bound method PipeConnection4Xi._check_closed of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003205CF8>>
            
        except:
            message2print = "{(Step 2/5z) yielded" + "; handle-yielded-by-recv-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                   
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_readable()
            message2print = "{(Step 3/5a) performed" + "; check_readable-performed-by-recv-ConnectionBase4Xi: " + str(self._check_readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: <bound method PipeConnection4Xi._check_readable of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000037E6390>>
            #TB: <bound method PipeConnection4Xi._check_readable of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003205CF8>>
            
        except:
            message2print = "{(Step 3/5z) yielded" + "; handle-yielded-by-recv-ConnectionBase4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) - NOTE THAT YOU CAN NOT TRY SEVERAL TIMES IN THIS LEVEL - LEAVE IT TO APP!   
        # -----------------------------------------------------------------------------------------
        buf2xi = None

        message2print = "{(Step 4/5b) performing" + "; self._recv_bytes-performing-by-recv-ConnectionBase4Xi: " + str(self._recv_bytes)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HP: <bound method PipeConnection4Xi._recv_bytes of <pipeconnection4xi.PipeConnection4Xi object at 0x00000000037E6390>>
        #TB: <bound method PipeConnection4Xi._recv_bytes of <pipeconnection4xi.PipeConnection4Xi object at 0x0000000003205CF8>>
        
        try:
                
            buf2xi = self._recv_bytes()
                    
            message2print = "{(Step 4/5c) got" + "; buf2xi-got-by-recv-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HP: <_io.BytesIO object at 0x00000000038349A8>
                    

        except:
            message2print = "{(Step 4/5z2) failed" + "; buf2xi-failed-by-recv-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                            
        

        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------        
        buffer2xi = None
        if buf2xi is None:
            message2print = "{(Step 5/5z1) warned" + "; buf2xi-warned-by-recv-ConnectionBase4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        else:    
            try:
                message2print = "{(Step 5/5a) performing" + "; getbuffer-of-buf2xi-performing-by-recv-ConnectionBase4Xi: " + str(buf2xi.getbuffer)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <built-in method getbuffer of _io.BytesIO object at 0x00000000038349A8>
                
                buffer2xi = buf2xi.getbuffer()
                
                message2print = "{(Step 5/5b) got" + "; buffer2xi-got-by-recv-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # <memory at 0x000000000395CCC8>
              
                
            except:
                message2print = "{(Step 5/5z2) failed" + "; buffer2xi-failed-by-recv-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 
                
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------           
        buffer2pickle = None
        try:
            buffer2pickle = pickle.loads(buffer2xi)
            
            message2print = "{(Step 5/5c) got" + "; buffer2pickle-got-by-recv-ConnectionBase4Xi: " + str(buffer2pickle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # [1, '(STEP 2A/33) OF HELLO DUPLEX PIPE4XI: HELLO B4XI, A4XI HERE !', None]
            # (STEP 3B/33) OF HELLO DUPLEX PIPE4XI - RECEIVED MESSAGE:  [1, '(STEP 2A/33) OF HELLO DUPLEX PIPE4XI: HELLO B4XI, A4XI HERE !', None]
            # 
        except:    
            message2print = "{(Step 5/5z2) failed" + "; buffer2xi-failed-by-recv-ConnectionBase4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #  
         
        message2print = "{(Step 5/5c) returning" + "; buffer2pickle-returning-by-recv-ConnectionBase4Xi: " + str(buffer2pickle)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
                  
        return buffer2pickle
        
    #</methodize4xi>
    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "poll-ConnectionBase4Xi" usage = "polls input">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def poll(self, timeout=0.0):
        '''
        method poll: polls input
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = 'poll'
        
        message2print = "{(Step 1/5a) started" + "; poll-ConnectionBase4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #HP: 06:14:42 01/17/13 Pacific Standard Time
        
        message2print = "{(Step 1/5b) passed" + "; timeout-passed-to-poll-ConnectionBase4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HP: 2 <-- looks like parameter was not given! 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/5)      
        # -----------------------------------------------------------------------------------------
        try:
            timeout = omega2timeout4xi(class2print, method2print)
            message2print = "{(Step 2/5a) set" + "; timeout-set-by-poll-ConnectionBase4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HP: 0.2
            
        except:
            timeout = 0.0
            message2print = "{(Step 2/5b) set" + "; timeout-set-by-poll-ConnectionBase4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_closed()
            
            message2print = "{(Step 3/5a) performed" + "; check-closed-of-connectionbase2xi-performed-by-poll-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        except:
            message2print = "{(Step 3/5z) yielded" + "; check-closed-of-connectionbase2xi-yielded-by-poll-ConnectionBase4Xi: " + str(self._check_closed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                   
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5)      
        # -----------------------------------------------------------------------------------------
        try:
            self._check_readable()
            
            message2print = "{(Step 4/5a) performed" + "; check-readable-of-connectionbase2xi-performed-by-poll-ConnectionBase4Xi: " + str(self._check_readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
        except:
            message2print = "{(Step 4/5z) yielded" + "; check-readable-of-connectionbase2xi-yielded-by-poll-ConnectionBase4Xi: " + str(self._check_readable)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------        
       
        message2print = "{(Step 5/5) returning" + "; poll-of-connectionbase2xi-returning-by-poll-ConnectionBase4Xi: " + str(self._poll)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        return self._poll(timeout)
        
    #</methodize4xi>
                        
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "enter-ConnectionBase4Xi" usage = "enter">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        method __enter__: enter
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/2)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = '__enter__'
        
        message2print = "{(Step 1/2a) started" + "; enter-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 2/2) returning" + "; enter-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  
        return       

        
    #</methodize4xi>    
                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "exit-ConnectionBase4Xi" usage = "exits">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def __exit__(self, exc_type, exc_value, exc_tb):
        '''
        method __exit__: exits
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)      
        # -----------------------------------------------------------------------------------------
        class2print = '_ConnectionBase4Xi'
        method2print = '__exit__'
        
        message2print = "{(Step 1/3a) started" + "; exit-ConnectionBase4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        message2print = "{(Step 1/3b) passed" + "; exc_type-passed-to-exit-ConnectionBase4Xi: " + str(exc_type) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3c) passed" + "; exc_value-passed-to-exit-ConnectionBase4Xi: " + str(exc_value) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{(Step 1/3d) passed" + "; exc_tb-passed-to-exit-ConnectionBase4Xi: " + str(exc_tb) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #                 
                        
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)      
        # -----------------------------------------------------------------------------------------        
        try:
            self.close()
            
            message2print = "{(Step 2/3a) performed" + "; close-of-connectionbase2xi-performed-by-exit-ConnectionBase4Xi: " + str(self.close)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
        except:
            message2print = "{(Step 2/3z) yielded" + "; close-yielded-by-exit-ConnectionBase4Xi: " + str(None)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #         
            
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------        
       
        message2print = "{(Step 3/3) returning" + "; exit-of-ConnectionBase4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #    
          
        return
        
    #</methodize4xi>  
    
                                                                                                                                                                
                                                                                                                                                                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ConnectionBase4Xi" usage = "senses _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = '_ConnectionBase4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-ConnectionBase4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; _ConnectionBase4Xi-sensing-by-sense-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-ConnectionBase4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; _ConnectionBase4Xi-sensed-by-sense-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-ConnectionBase4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
      


        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ConnectionBase4Xi" usage = "reasons _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons _ConnectionBase4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = '_ConnectionBase4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-ConnectionBase4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; _ConnectionBase4Xi-reasoning-by-reason-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-ConnectionBase4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{(Step 4/5) reasoning" + "; _ConnectionBase4Xi-reasoning-by-reason-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-ConnectionBase4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
         

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ConnectionBase4Xi" usage = "aces _ConnectionBase4Xi class objects">
    #<note4xi note = "[Rel 7.21]"/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces _ConnectionBase4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = '_ConnectionBase4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-ConnectionBase4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; _ConnectionBase4Xi-acing-by-ace-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-ConnectionBase4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; _ConnectionBase4Xi-acing-by-ace-of-ConnectionBase4Xi: " + str(_ConnectionBase4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-ConnectionBase4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of _connectionbase4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

